@@ -26,7 +26,7 @@ use constant TBZ            => 'tbz';
 
 use vars qw[$VERSION $PREFER_BIN $PROGRAMS $WARN $DEBUG];
 
-$VERSION        = '0.11_02';
+$VERSION        = '0.15_01';
 $PREFER_BIN     = 0;
 $WARN           = 1;
 $DEBUG          = 0;
@@ -37,7 +37,7 @@ local $Params::Check::VERBOSE = $Params::Check::VERBOSE = 1;
 
 =head1 NAME
 
-Archive::Extract -- A generic archive extracting mechanism
+Archive::Extract - A generic archive extracting mechanism
 
 =head1 SYNOPSIS
 
@@ -116,8 +116,8 @@ my $Mapping = {
     };
 
     ### build accesssors ###
-    for my $method( keys %$tmpl,
-                    qw[_extractor _gunzip_to files extract_path]
+    for my $method( keys %$tmpl, qw[_extractor _gunzip_to files extract_path],
+                    qw[_error_msg _error_msg_long]
     ) {
         no strict 'refs';
         *$method = sub {
@@ -434,9 +434,8 @@ sub bin_bunzip2 { return $PROGRAMS->{'bunzip2'} if $PROGRAMS->{'bunzip2'} }
 sub _untar {
     my $self = shift;
 
-    ### no bzip2 support in A::T yet
-    my   @methods = qw[_untar_bin];
-    push @methods,  qw[_untar_at] unless $self->is_tbz;
+    ### bzip2 support in A::T via IO::Uncompress::Bzip2
+    my   @methods = qw[_untar_at _untar_bin];
          @methods = reverse @methods unless $PREFER_BIN;
 
     for my $method (@methods) {
@@ -491,10 +490,10 @@ sub _untar_bin {
                             $self->archive, $buffer ));
         }
 
-        unless( $buffer ) {
-            $self->_error(loc("No buffer captured, unable to tell ".
-                              "extracted files or extraction dir for '%1'",
-                              $self->archive));
+        ### no buffers available?
+        if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+            $self->_error( $self->_no_buffer_files( $self->archive ) );
+        
         } else {
             ### if we're on solaris we /might/ be using /bin/tar, which has
             ### a weird output format... we might also be using
@@ -561,6 +560,9 @@ sub _untar_at {
         }
     }
 
+    ### we might pass it a filehandle if it's a .tbz file..
+    my $fh_to_read = $self->archive;
+
     ### we will need Compress::Zlib too, if it's a tgz... and IO::Zlib
     ### if A::T's version is 0.99 or higher
     if( $self->is_tgz ) {
@@ -576,11 +578,28 @@ sub _untar_at {
                                 "install it as soon as possible.", $which));
 
         }
+    } elsif ( $self->is_tbz ) {
+        my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+        unless( can_load( modules => $use_list ) ) {
+            return $self->_error(loc(
+                    "You do not have '%1' installed - Please " .
+                    "install it as soon as possible.", 
+                     'IO::Uncompress::Bunzip2'));
+        }
+
+        my $bz = IO::Uncompress::Bunzip2->new( $self->archive ) or
+            return $self->_error(loc("Unable to open '%1': %2",
+                            $self->archive,
+                            $IO::Uncompress::Bunzip2::Bunzip2Error));
+
+        $fh_to_read = $bz;
     }
 
     my $tar = Archive::Tar->new();
 
-    unless( $tar->read( $self->archive, $self->is_tgz ) ) {
+    ### only tell it it's compressed if it's a .tgz, as we give it a file
+    ### handle if it's a .tbz
+    unless( $tar->read( $fh_to_read, ( $self->is_tgz ? 1 : 0 ) ) ) {
         return $self->_error(loc("Unable to read '%1': %2", $self->archive,
                                     $Archive::Tar::error));
     }
@@ -668,9 +687,9 @@ sub _gunzip_bin {
                                     $self->archive, $buffer));
     }
 
-    unless( $buffer ) {
-        $self->_error(loc("No buffer captured, unable to get content for '%1'",
-                          $self->archive));
+    ### no buffers available?
+    if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+        $self->_error( $self->_no_buffer_content( $self->archive ) );
     }
 
     print $fh $buffer if defined $buffer;
@@ -744,7 +763,7 @@ sub _unzip_bin {
     ### first, get the files.. it must be 2 different commands with 'unzip' :(
     {   my $cmd = [ $self->bin_unzip, '-Z', '-1', $self->archive ];
 
-        my $buffer;
+        my $buffer = '';
         unless( scalar run( command => $cmd,
                             verbose => $DEBUG,
                             buffer  => \$buffer )
@@ -753,10 +772,9 @@ sub _unzip_bin {
                                         $self->archive, $buffer));
         }
 
-        unless( $buffer ) {
-            $self->_error(loc("No buffer captured, unable to tell extracted ".
-                              "files or extraction dir for '%1'",
-                              $self->archive));
+        ### no buffers available?
+        if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+            $self->_error( $self->_no_buffer_files( $self->archive ) );
 
         } else {
             $self->files( [split $/, $buffer] );
@@ -870,8 +888,8 @@ sub __get_extract_dir {
 sub _bunzip2 {
     my $self = shift;
 
-    my @methods = qw[_bunzip2_bin];
-    #   @methods = reverse @methods if $PREFER_BIN;
+    my @methods = qw[_bunzip2_cz2 _bunzip2_bin];
+       @methods = reverse @methods if $PREFER_BIN;
 
     for my $method (@methods) {
         $self->_extractor($method) && return 1 if $self->$method();
@@ -903,11 +921,11 @@ sub _bunzip2_bin {
                                     $self->archive, $buffer));
     }
 
-    unless( $buffer ) {
-        $self->_error(loc("No buffer captured, unable to get content for '%1'",
-                          $self->archive));
+    ### no buffers available?
+    if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+        $self->_error( $self->_no_buffer_content( $self->archive ) );
     }
-
+    
     print $fh $buffer if defined $buffer;
 
     close $fh;
@@ -919,6 +937,57 @@ sub _bunzip2_bin {
     return 1;
 }
 
+### using cz2, the compact versions... this we use mainly in archive::tar
+### extractor..
+# sub _bunzip2_cz1 {
+#     my $self = shift;
+# 
+#     my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+#     unless( can_load( modules => $use_list ) ) {
+#         return $self->_error(loc("You do not have '%1' installed - Please " .
+#                         "install it as soon as possible.", 'IO::Uncompress::Bunzip2'));
+#     }
+# 
+#     my $bz = IO::Uncompress::Bunzip2->new( $self->archive ) or
+#                 return $self->_error(loc("Unable to open '%1': %2",
+#                             $self->archive,
+#                             $IO::Uncompress::Bunzip2::Bunzip2Error));
+# 
+#     my $fh = FileHandle->new('>'. $self->_gunzip_to) or
+#         return $self->_error(loc("Could not open '%1' for writing: %2",
+#                             $self->_gunzip_to, $! ));
+# 
+#     my $buffer;
+#     $fh->print($buffer) while $bz->read($buffer) > 0;
+#     $fh->close;
+# 
+#     ### set what files where extract, and where they went ###
+#     $self->files( [$self->_gunzip_to] );
+#     $self->extract_path( File::Spec->rel2abs(cwd()) );
+# 
+#     return 1;
+# }
+
+sub _bunzip2_cz2 {
+    my $self = shift;
+
+    my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+    unless( can_load( modules => $use_list ) ) {
+        return $self->_error(loc("You do not have '%1' installed - Please " .
+                        "install it as soon as possible.", 'IO::Uncompress::Bunzip2'));
+    }
+
+    IO::Uncompress::Bunzip2::bunzip2($self->archive => $self->_gunzip_to)
+        or return $self->_error(loc("Unable to uncompress '%1': %2",
+                            $self->archive,
+                            $IO::Uncompress::Bunzip2::Bunzip2Error));
+
+    ### set what files where extract, and where they went ###
+    $self->files( [$self->_gunzip_to] );
+    $self->extract_path( File::Spec->rel2abs(cwd()) );
+
+    return 1;
+}
 
 
 #################################
@@ -927,31 +996,39 @@ sub _bunzip2_bin {
 #
 #################################
 
-### Error handling, the way Archive::Tar does it ###
-{
-    my $error       = '';
-    my $longmess    = '';
-
-    sub _error {
-        my $self    = shift;
-        $error      = shift;
-        $longmess   = Carp::longmess($error);
-
-        ### set Archive::Tar::WARN to 0 to disable printing
-        ### of errors
-        if( $WARN ) {
-            carp $DEBUG ? $longmess : $error;
-        }
-
-        return;
+sub _error {
+    my $self    = shift;
+    my $error   = shift;
+    
+    $self->_error_msg( $error );
+    $self->_error_msg_long( Carp::longmess($error) );
+    
+    ### set Archive::Tar::WARN to 0 to disable printing
+    ### of errors
+    if( $WARN ) {
+        carp $DEBUG ? $self->_error_msg_long : $self->_error_msg;
     }
 
-    sub error {
-        my $self = shift;
-        return shift() ? $longmess : $error;
-    }
+    return;
 }
 
+sub error {
+    my $self = shift;
+    return shift() ? $self->_error_msg_long : $self->_error_msg;
+}
+
+sub _no_buffer_files {
+    my $self = shift;
+    my $file = shift or return;
+    return loc("No buffer captured, unable to tell ".
+               "extracted files or extraction dir for '%1'", $file);
+}
+
+sub _no_buffer_content {
+    my $self = shift;
+    my $file = shift or return;
+    return loc("No buffer captured, unable to get content for '%1'", $file);
+}
 1;
 
 =pod
